@@ -174,12 +174,20 @@ extern "C" {
 #  endif
 #endif
 
+#if MYSQL_VERSION_ID >= 50612 && !defined(MRN_MARIADB_P)
+#  define MRN_HAVE_POINT_XY
+#endif
+
 #if defined(_WIN32) || defined(_WIN64)
 #  define MRN_API __declspec(dllexport)
 #else
 #  define MRN_API
 #endif
 
+#if (defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 100000)
+#  define MRN_HANDLER_START_BULK_INSERT_HAS_FLAGS
+#endif
+
 class ha_mroonga;
 
 /* structs */
@@ -418,7 +426,11 @@ public:
                              HANDLER_BUFFER *buffer);
   int read_multi_range_next(KEY_MULTI_RANGE **found_range_p);
 #endif // MRN_HANDLER_HAVE_MULTI_RANGE_READ
+#ifdef MRN_HANDLER_START_BULK_INSERT_HAS_FLAGS
+  void start_bulk_insert(ha_rows rows, uint flags);
+#else
   void start_bulk_insert(ha_rows rows);
+#endif
   int end_bulk_insert();
   int delete_all_rows();
 #ifdef MRN_HANDLER_HAVE_TRUNCATE
@@ -628,6 +640,9 @@ private:
                                             grn_id record_id);
   void storage_store_fields_by_index(uchar *buf);
 
+  int storage_encode_key_normalize_min_sort_chars(Field *field,
+                                                  uchar *buf,
+                                                  uint size);
   int storage_encode_key_fixed_size_string(Field *field, const uchar *key,
                                            uchar *buf, uint *size);
   int storage_encode_key_variable_size_string(Field *field, const uchar *key,
@@ -708,6 +723,7 @@ private:
                              grn_obj *grn_table, MRN_SHARE *tmp_share);
   int close_databases();
   void ensure_database_directory();
+  int ensure_normalizers_register();
   int ensure_database_create(const char *name);
   int ensure_database_open(const char *name);
   int wrapper_delete_table(const char *name, MRN_SHARE *tmp_share,
@@ -740,8 +756,13 @@ private:
                                      enum thr_lock_type lock_type);
   int wrapper_external_lock(THD *thd, int lock_type);
   int storage_external_lock(THD *thd, int lock_type);
+#ifdef MRN_HANDLER_START_BULK_INSERT_HAS_FLAGS
+  void wrapper_start_bulk_insert(ha_rows rows, uint flags);
+  void storage_start_bulk_insert(ha_rows rows, uint flags);
+#else
   void wrapper_start_bulk_insert(ha_rows rows);
   void storage_start_bulk_insert(ha_rows rows);
+#endif
   int wrapper_end_bulk_insert();
   int storage_end_bulk_insert();
   bool wrapper_is_target_index(KEY *key_info);
@@ -856,6 +877,32 @@ private:
   FT_INFO *storage_ft_init_ext(uint flags, uint key_nr, String *key);
   void generic_ft_init_ext_add_conditions_fast_order_limit(
       struct st_mrn_ft_info *info, grn_obj *expression);
+  bool generic_ft_init_ext_parse_pragma_d(struct st_mrn_ft_info *info,
+                                          const char *keyword,
+                                          uint keyword_length,
+                                          grn_operator *default_operator,
+                                          uint *consumed_keyword_length);
+  bool generic_ft_init_ext_parse_pragma_w(struct st_mrn_ft_info *info,
+                                          const char *keyword,
+                                          uint keyword_length,
+                                          grn_obj *index_column,
+                                          grn_obj *match_columns,
+                                          uint *consumed_keyword_length,
+                                          grn_obj *tmp_objects);
+  grn_rc generic_ft_init_ext_prepare_expression_in_boolean_mode(
+    struct st_mrn_ft_info *info,
+    String *key,
+    grn_obj *index_column,
+    grn_obj *match_columns,
+    grn_obj *expression,
+    grn_obj *tmp_objects);
+  grn_rc generic_ft_init_ext_prepare_expression_in_normal_mode(
+    struct st_mrn_ft_info *info,
+    String *key,
+    grn_obj *index_column,
+    grn_obj *match_columns,
+    grn_obj *expression,
+    grn_obj *tmp_objects);
   struct st_mrn_ft_info *generic_ft_init_ext_select(uint flags,
                                                     uint key_nr,
                                                     String *key);
@@ -984,6 +1031,7 @@ private:
   int storage_optimize(THD* thd, HA_CHECK_OPT* check_opt);
   bool wrapper_is_fatal_error(int error_num, uint flags);
   bool storage_is_fatal_error(int error_num, uint flags);
+  bool wrapper_is_comment_changed(TABLE *table1, TABLE *table2);
   bool wrapper_check_if_incompatible_data(HA_CREATE_INFO *create_info,
                                           uint table_changes);
   bool storage_check_if_incompatible_data(HA_CREATE_INFO *create_info,
