@@ -74,4 +74,13 @@
 #  define init_alloc_root(PTR, SZ1, SZ2, FLAG) init_alloc_root(PTR, SZ1, SZ2)
 #endif
 
+#if MYSQL_VERSION_ID < 100002 || !defined(MRN_MARIADB_P)
+#  define GTS_TABLE 0
+#endif
+
+/* For MySQL 5.1. MySQL 5.1 doesn't have FN_LIBCHAR2. */
+#ifndef FN_LIBCHAR2
+#  define FN_LIBCHAR2 FN_LIBCHAR
+#endif
+
 #endif /* _mrn_mysql_compat_h */
